--
-- Created by IntelliJ IDEA.
-- User: Orion
-- Date: 12/3/12
-- Time: 10:10 PM
-- To change this template use File | Settings | File Templates.
--

local MountDatabase = { }

function CreateMountDatabase()
    MountDatabase:LoadRaw()
    MountDatabase:BuildDatabase()

    return MountDatabase
end

function MountDatabase:FindActiveMountSpell()
    for i = 1, self:GetNumCompanions("MOUNT") do
        local _, _, spellID, _, active, _ = self:GetCompanionInfo("MOUNT", i);

        if (active) then
            return spellID
        end
    end

    return nil
end

function MountDatabase:GetNumCompanions(type)
    if (type == "PET") then
        return GetNumCompanions("PET")
    end

    if (type == "MOUNT") then
        return #self.data
    end
end

function MountDatabase:GetCompanionInfo(type, index)
	if  (type == "PET") then
		return GetCompanionInfo("PET", index)
	end
		 
	if (type == "MOUNT") then
        local entry = self.data[index]

        local _, active

        if (entry.originalIndex) then
            _, _, _, _, active = GetCompanionInfo("MOUNT", entry.originalIndex)
        end

		return nil, entry.creatureName, entry.spellID, entry.icon, active, nil, entry.owned, entry.displayInfoID, entry.originalIndex
	end

	return nil
end

function MountDatabase:BuildDatabase()
    self.source = {"drop","quest","vendor","profession","achievement","faction","event","promotion","tcg","store","other"}
    self.quality = {"common","uncommon","rare","epic","legendary"}
    self.type = {"ground","flying","aquatic"}

    -- expand raw data into data

    self.data = { }

    for _, entry in ipairs(self.raw) do
        local name, _, icon = GetSpellInfo(entry[1])

        local expanded = {
            spellID = entry[1],
            creatureID = entry[2],
            itemID = entry[3],
            displayInfoID = entry[4],
            sourceIndex = entry[5],
            qualityIndex = entry[6],
            typeIndex = entry[7],
            creatureName = name,
            icon = icon,
            owned = false,
        }

        table.insert(self.data, expanded)
    end

    -- build indexes

    self.index = { }

    for _, entry in ipairs(self.data) do
        self.index[entry.spellID] = entry
    end

    local _, creatureName = GetCompanionInfo("MOUNT", 1)

    -- add user mounts that might not be in our system. We need to do this to get a proper GetNumCompanions count

    for index = 1, GetNumCompanions("MOUNT") do
        local creatureID, creatureName, spellID, icon, active, mountFlags = GetCompanionInfo("MOUNT", index)

        local entry = self.index[spellID]

        if (entry == nil) then
            entry = { -- minimal entry just to fill a spot
                spellID = spellID,
                creatureID = creatureID,
                creatureName = creatureName,
                icon = icon,
            }

            table.insert(self.data, entry) -- add to data

            self.index[entry.spellID] = entry -- add to index
        end

        entry.owned = true
        entry.originalIndex = index

    end

    -- sort the raw data so that it shows up in the right order initially (we don't want to have to resort the
    -- results if we don't have to.

    table.sort(self.data, function(a, b) return a.creatureName < b.creatureName end)

end

function MountDatabase:LoadRaw()
    self.raw = { --spellID, creatureID, itemID, displayID, Source[source], Quality[quality], Type[type]}
        {26656,15711,21176,15676,11,5,1},
        {60025,32158,44178,25836,5,4,2},
        {127180,65011,87785,43708,11,4,1},
        {98204,53276,69747,38261,1,4,1},
        {96503,52185,68825,37800,9,4,2},
        {43688,24379,33809,22464,11,4,1},
        {123886,63502,85262,43090,6,4,1},
        {16056,10322,12302,9695,11,4,1},
        {66906,35179,47179,28919,3,4,1},
        {63844,33857,45725,22471,3,4,2},
        {67466,35445,47180,28918,3,4,1},
        {61230,32336,44690,27914,3,4,2},
        {60116,32207,44226,27821,3,4,1},
        {60114,32206,44225,27820,3,4,1},
        {96491,52172,68823,14341,1,4,1},
        {61229,32335,44689,27913,3,4,2},
        {132117,66971,90710,45521,3,4,2},
        {40192,18545,32458,17890,1,4,2},
        {127170,64996,87777,46087,11,4,2},
        {123992,63552,85430,41989,6,4,2},
        {59567,31694,43952,27785,1,4,2},
        {41514,23456,32858,21521,3,4,2},
        {127174,65006,87781,43704,6,4,1},
        {118089,60941,81354,41711,6,4,1},
        {51412,28363,49282,25335,9,4,1},
        {71342,38207,50250,30989,7,4,2},
        {22719,14334,29465,14372,3,4,1},
        {59650,31778,43986,25831,1,4,2},
        {16055,7322,12303,9991,11,4,1},
        {59976,31912,44164,28040,11,4,2},
        {17461,4780,13328,2784,11,4,1},
        {130138,66177,89391,44836,6,4,1},
        {127209,65015,87786,43709,11,4,1},
        {60118,32203,44223,27818,5,4,1},
        {60119,32205,44224,27819,5,4,1},
        {48027,26439,35906,23928,3,4,1},
        {22718,14333,29466,14348,3,4,1},
        {59788,31850,44077,27245,3,4,1},
        {59785,31849,43956,27247,3,4,1},
        {22720,14335,29467,14577,3,4,1},
        {22721,14330,29472,14388,3,4,1},
        {22717,14332,29468,14337,3,4,1},
        {22723,14336,29471,14330,3,4,1},
        {22724,14329,29469,14334,3,4,1},
        {107842,57226,77067,39561,1,4,2},
        {74856,40165,54069,31803,9,4,2},
        {127220,65018,87789,43712,3,4,1},
        {72808,38778,51954,31156,5,4,2},
        {61996,31239,44843,27525,5,4,2},
        {59568,31695,43953,25832,1,4,2},
        {59996,32151,44151,28041,1,4,2},
        {39803,22978,32319,21156,6,4,2},
        {129934,66063,89307,43900,6,4,1},
        {59569,31717,43951,25833,1,4,2},
        {88748,47652,63044,35136,6,4,1},
        {130086,66150,89362,44807,6,4,1},
        {127213,65016,87787,43710,11,4,1},
        {58615,31124,43516,27507,11,4,2},
        {124550,63816,85785,38757,11,4,2},
        {75614,-1,54811,10,4,2,31958},
        {43927,24488,33999,22473,6,4,2},
        {41515,23460,32859,21525,3,4,2},
        {39315,22510,31830,21073,6,4,1},
        {34896,20072,29102,19375,6,4,1},
        {97560,52807,69230,38046,5,4,2},
        {102514,54903,72582,38972,11,4,2},
        {127156,64981,87769,41592,5,4,2},
        {73313,39046,52200,25279,2,4,1},
        {129552,65851,89154,44633,3,4,2},
        {123160,63038,84728,42837,11,4,1},
        {127271,65030,87791,43713,11,4,1},
        {68188,35878,49098,29938,5,4,1},
        {68187,35876,49096,29937,5,4,1},
        {88990,47841,63125,37145,5,4,2},
        {39316,22511,28915,21074,3,4,1},
        {34790,20149,29228,19303,3,4,1},
        {103081,55188,73766,39060,3,4,1},
        {63635,33299,45593,29261,3,4,1},
        {63637,33298,45591,29256,3,4,1},
        {64927,34225,46708,25511,11,4,2},
        {88335,47353,62901,35757,5,4,2},
        {88742,47648,63040,35553,1,4,2},
        {88744,47646,63041,35755,1,4,2},
        {88741,47647,63039,35754,6,4,2},
        {132118,66972,90711,45520,3,4,2},
        {63639,33416,45590,29255,3,4,1},
        {110039,58166,78919,39229,1,4,2},
        {113120,59072,79771,40568,11,4,2},
        {36702,21354,30480,19250,1,4,1},
        {101542,54395,71665,38783,1,4,1},
        {97359,52672,69213,38018,5,4,2},
        {63643,33414,45597,29257,3,4,1},
        {84751,45338,60954,34410,4,4,1},
        {17460,4778,13329,2787,11,4,1},
        {23509,14744,19029,14776,3,4,1},
        {75596,40533,54797,28063,4,4,2},
        {65439,34425,46171,25593,11,4,2},
        {63638,33301,45589,28571,3,4,1},
        {87091,46755,62462,35250,3,4,1},
        {123993,63553,85429,41991,6,4,2},
        {90621,48632,62298,36213,5,4,1},
        {127176,65007,87782,43705,6,4,1},
        {127278,65033,87794,43716,11,4,1},
        {135416,68771,93168,6,4,2,-1},
        {135418,68773,93169,6,4,2,-1},
        {61467,31861,44083,27240,1,4,1},
        {61465,31862,43959,27241,1,4,1},
        {122708,62809,84101,42703,3,4,1},
        {136163,69067,93385,2,4,2,-1},
        {61470,31858,43961,27242,6,4,1},
        {61469,31857,44086,27239,6,4,1},
        {136164,69066,93386,2,4,2,-1},
        {127295,65072,87802,43723,3,4,1},
        {127302,65074,87803,43724,3,4,1},
        {35713,20848,29745,19871,3,4,1},
        {49379,27707,37828,24757,7,4,1},
        {127308,65076,87804,43725,3,4,1},
        {23249,14549,18794,14578,3,4,1},
        {65641,34558,46755,28556,6,4,1},
        {23248,14550,18795,14579,3,4,1},
        {127293,65071,87801,43722,3,4,1},
        {35712,20849,29746,19873,3,4,1},
        {127310,65078,87805,43726,3,4,1},
        {35714,20850,29747,19872,3,4,1},
        {120822,62106,82811,42352,3,4,1},
        {65637,34551,46756,28606,6,4,1},
        {23247,14542,18793,14349,3,4,1},
        {18991,12151,15292,12245,11,4,1},
        {61294,32562,44707,28053,11,4,2},
        {39798,22958,32314,21152,6,4,2},
        {129932,66060,89305,44759,6,4,1},
        {17465,11156,13334,10720,3,4,1},
        {88750,47654,63046,35135,1,4,1},
        {127216,65017,87788,43711,3,4,1},
        {48025,27152,37012,25159,7,4,2},
        {110051,-1,78924,10,4,2,40029},
        {127169,64993,87776,43697,11,4,2},
        {127161,64990,87773,43692,5,4,2},
        {127164,64991,87774,43693,11,4,2},
        {127165,64992,87775,43695,11,4,2},
        {127158,64986,87771,43689,11,4,2},
        {59797,31854,44080,27246,6,4,1},
        {59799,31855,43958,27248,6,4,1},
        {72807,38695,51955,31154,5,4,2},
        {17459,11150,13327,10666,11,4,1},
        {124659,63831,85870,43254,11,4,2},
        {72286,38260,50818,31007,1,4,2},
        {63956,33892,45801,28953,5,4,2},
        {63636,33408,45586,29258,3,4,1},
        {17450,7706,13317,6471,11,4,1},
        {113199,59101,79802,40590,6,4,2},
        {133023,67437,91802,5,4,2,-1},
        {127274,65032,87793,43714,11,4,1},
        {120043,61652,82453,42185,4,4,2},
        {127178,65010,87784,43707,11,4,1},
        {93644,51195,67107,37138,5,4,1},
        {107845,57228,77069,39563,1,4,2},
        {65917,34655,49290,29344,9,4,1},
        {61309,33030,44558,28060,4,4,2},
        {55531,29929,41508,25871,4,4,1},
        {60424,32286,44413,25870,4,4,1},
        {44744,24743,34092,22620,11,4,2},
        {63796,33848,45693,28890,1,4,2},
        {93623,50467,68008,37231,9,4,2},
        {16084,7704,8586,6469,11,4,1},
        {121820,62454,83086,42498,8,4,2},
        {66846,35169,47101,29754,3,4,1},
        {127154,64977,87768,41990,2,4,2},
        {41513,23455,32857,21520,3,4,2},
        {69395,36837,49636,30346,1,4,2},
        {127272,65031,87792,43715,11,4,1},
        {63640,33409,45595,29260,3,4,1},
        {16082,306,12354,2408,11,4,1},
        {118737,61201,81559,41903,5,4,2},
        {130985,66661,89785,45271,5,4,2},
        {88718,47631,63042,35740,1,4,2},
        {60021,32156,44175,28042,11,4,2},
        {97493,52748,69224,38031,1,4,2},
        {41516,23458,32860,21523,3,4,2},
        {39801,22975,32316,21155,6,4,2},
        {23246,14558,18791,10721,3,4,1},
        {66090,33840,46815,28888,6,4,1},
        {41252,23408,32768,21473,1,4,1},
        {61997,32944,44842,28402,5,4,2},
        {59570,31697,43955,25835,6,4,2},
        {59961,31902,44160,28044,5,4,2},
        {39800,22976,32317,21158,6,4,2},
        {129935,66062,89306,44757,6,4,1},
        {22722,14331,29470,10719,3,4,1},
        {16080,4270,12330,2326,11,4,1},
        {127177,65009,87783,43706,6,4,1},
        {67336,35362,47840,29794,11,4,2},
        {30174,17266,23720,17158,9,4,1},
        {17481,30542,13335,10718,1,4,1},
        {63963,33904,45802,28954,5,4,2},
        {101821,54498,71954,38755,11,4,2},
        {93326,50269,65891,35750,4,4,2},
        {97581,52813,69228,38048,9,4,1},
        {66087,35147,46813,22474,6,4,2},
        {39802,22977,32318,21157,6,4,2},
        {39317,22512,31832,21075,6,4,1},
        {34898,20152,29104,19378,6,4,1},
        {63642,33418,45596,29262,3,4,1},
        {130965,66645,89783,45264,11,4,1},
        {107516,57156,76889,39546,11,4,2},
        {92231,49487,64998,37160,6,4,1},
        {107517,57157,76902,39547,11,4,2},
        {92232,49488,64999,37159,6,4,1},
        {23510,14745,19030,14777,3,4,1},
        {63232,33297,45125,28912,3,4,1},
        {98718,53270,67151,34955,1,4,3},
        {66088,35148,46814,29696,6,4,2},
        {66091,33841,46816,28889,6,4,1},
        {68057,35808,49044,29284,11,4,1},
        {32242,18406,25473,17759,3,4,2},
        {23241,14545,18788,14339,3,4,1},
        {43900,24368,33977,22350,7,4,1},
        {23238,14546,18786,14347,3,4,1},
        {23229,14561,18777,14583,3,4,1},
        {23250,14540,18796,14573,3,4,1},
        {65646,34555,46757,14335,6,4,1},
        {102346,54740,72140,1281,3,4,1},
        {23221,14556,18766,14331,3,4,1},
        {23239,14548,18787,14576,3,4,1},
        {65640,34557,46758,29043,6,4,1},
        {23252,14541,18798,14574,3,4,1},
        {32290,18375,25528,17703,3,4,2},
        {35025,20224,29223,19484,3,4,1},
        {23225,14553,18772,14374,3,4,1},
        {32295,18378,25531,17720,3,4,2},
        {68056,35809,49046,29283,11,4,1},
        {102350,54742,72146,1961,3,4,1},
        {23219,14555,18767,14332,3,4,1},
        {65638,34550,46759,14333,6,4,1},
        {103196,55273,73839,39095,3,4,1},
        {37015,21510,30609,20344,11,4,2},
        {23242,14543,18789,14344,3,4,1},
        {23243,14544,18790,14342,3,4,1},
        {23227,14559,18776,14582,3,4,1},
        {33660,19281,28936,18697,3,4,1},
        {32292,18362,25529,17717,3,4,2},
        {35027,20223,29224,19482,3,4,1},
        {65644,34549,46760,14343,6,4,1},
        {32297,18379,25533,17721,3,4,2},
        {24242,15090,19872,15289,11,4,1},
        {32289,18376,25527,17718,3,4,2},
        {65639,34556,46761,28607,6,4,1},
        {32246,18377,25477,17719,3,4,2},
        {101573,54423,71718,17011,11,4,1},
        {42777,24004,49284,21974,9,4,1},
        {102349,54741,72145,16992,3,4,1},
        {23338,14602,18902,14632,3,4,1},
        {23251,14539,18797,14575,3,4,1},
        {65643,34554,46762,28612,6,4,1},
        {35028,20225,34129,20359,3,4,1},
        {46628,26131,35513,19483,1,4,1},
        {23223,14552,18773,14376,3,4,1},
        {23240,14547,18785,14346,3,4,1},
        {23228,14560,18778,14338,3,4,1},
        {134573,68288,92724,46729,11,4,2},
        {23222,14551,18774,14377,3,4,1},
        {32296,18380,25532,17722,3,4,2},
        {49322,27684,37719,24745,11,4,1},
        {96499,52178,68824,37799,1,4,1},
        {24252,15104,19902,15290,11,4,1},
        {88749,47653,63045,35134,6,4,1},
        {39318,22513,31833,21077,6,4,1},
        {34899,20150,29105,19376,6,4,1},
        {18992,12148,15293,12242,11,4,1},
        {63641,33300,45592,29259,3,4,1},
        {129918,66051,89304,43686,6,4,2},
        {124408,63766,85666,43562,5,4,2},
        {132036,67145,90655,45797,1,4,2},
        {60002,32153,44168,28045,1,4,2},
        {61425,32633,44235,27237,3,4,1},
        {61447,32640,44234,27238,3,4,1},
        {44151,24654,34061,22720,4,4,2},
        {65642,34553,46763,14375,6,4,1},
        {59571,31698,43954,27796,1,4,2},
        {107844,57227,77068,39562,5,4,2},
        {107203,56921,76755,39530,11,4,2},
        {92155,51152,64883,15672,4,4,1},
        {49193,27637,37676,24725,11,4,2},
        {64659,34156,46102,29102,2,4,1},
        {41517,23457,32861,21522,3,4,2},
        {101282,54335,71339,38756,11,4,2},
        {100332,53885,70909,38668,5,4,1},
        {100333,53985,70910,38607,5,4,1},
        {41518,23459,32862,21524,3,4,2},
        {132119,66970,90712,45522,3,4,2},
        {60024,32157,44177,28043,5,4,2},
        {88746,47651,63043,35751,1,4,2},
        {88331,47352,62900,35551,5,4,2},
        {15779,10179,13326,9474,11,4,1},
        {54753,29596,43962,28428,11,4,1},
        {102488,54879,72575,37204,11,4,1},
        {130137,66176,89390,44837,6,4,1},
        {39319,22514,31835,21076,6,4,1},
        {123182,63042,84753,41089,11,4,1},
        {65645,34552,46764,28605,6,4,1},
        {16083,305,12353,2410,11,4,1},
        {34897,20151,29103,19377,6,4,1},
        {98727,-1,69846,10,4,2,38260},
        {54729,29582,40775,28108,3,4,2},
        {16081,359,12351,1166,11,4,1},
        {17229,11021,13086,10426,3,4,1},
        {59791,31851,44230,27243,3,4,1},
        {59793,31852,44231,27244,3,4,1},
        {74918,40191,54068,31721,9,4,1},
        {71810,38361,50435,31047,11,4,2},
        {46199,26164,49286,23647,9,4,2},
        {75973,40725,54860,31992,11,4,2},
        {75207,40054,54465,34956,2,3,3},
        {58983,31319,43599,27567,8,3,1},
        {127286,65058,87795,43717,3,3,1},
        {35022,20222,29221,19478,3,3,1},
        {64977,34238,46308,29130,3,3,1},
        {470,308,2411,2402,3,3,1},
        {64658,356,46099,207,3,3,1},
        {127287,65060,87796,43718,3,3,1},
        {35020,20220,29220,19480,3,3,1},
        {10969,7749,8595,6569,3,3,1},
        {25953,15666,21218,15672,1,3,1},
        {17463,11154,13332,10671,3,3,1},
        {32244,18364,25475,17700,3,3,2},
        {43899,23588,33976,22265,7,3,1},
        {127288,65061,87797,43719,3,3,1},
        {34406,19658,28481,17063,3,3,1},
        {458,284,5656,2404,3,3,1},
        {18990,11689,15290,11641,3,3,1},
        {6899,4779,5872,2785,3,3,1},
        {17464,11155,13333,10672,3,3,1},
        {6654,4272,5668,2328,3,3,1},
        {6648,4269,5655,2405,3,3,1},
        {126507,64715,87250,43637,4,3,2},
        {6653,4271,5665,2327,3,3,1},
        {32239,18357,25471,17694,3,3,2},
        {8395,6075,8588,4806,3,3,1},
        {17458,10178,13325,9475,11,3,1},
        {61451,33029,44554,28082,4,3,2},
        {44153,24653,34060,22719,4,3,2},
        {126508,64716,87251,43638,4,3,2},
        {87090,46754,62461,35249,3,3,1},
        {32235,18360,25470,17697,3,3,2},
        {35710,20846,29744,19869,3,3,1},
        {18989,12149,15277,12246,3,3,1},
        {6777,4710,5864,2736,3,3,1},
        {120395,61809,82765,42250,3,3,1},
        {17453,11147,13321,10661,3,3,1},
        {26056,15715,21323,15679,1,3,1},
        {32245,18365,25476,17701,3,3,2},
        {121837,62461,83088,42502,4,3,2},
        {103195,55272,73838,39096,3,3,1},
        {472,307,2414,2409,3,3,1},
        {127289,65063,87799,43720,3,3,1},
        {35711,20847,29743,19870,3,3,1},
        {35018,20217,29222,19479,3,3,1},
        {127290,65065,87800,43721,3,3,1},
        {130092,66151,89363,44808,6,3,2},
        {34795,19280,28927,18696,3,3,1},
        {10873,7739,8563,9473,3,3,1},
        {26054,15716,21321,15681,1,3,1},
        {17462,11153,13331,10670,3,3,1},
        {121838,62458,83087,42499,4,3,2},
        {121836,62459,83090,42500,4,3,2},
        {64731,34187,46109,29161,4,3,1},
        {32240,18359,25472,17696,3,3,2},
        {42776,24003,49283,21973,9,3,1},
        {10789,7687,8632,6444,3,3,1},
        {66847,35168,47100,29755,3,3,1},
        {8394,6074,8631,6080,3,3,1},
        {10793,7690,8629,6448,3,3,1},
        {121839,62460,83089,42501,4,3,2},
        {32243,18363,25474,17699,3,3,2},
        {580,358,1132,247,3,3,1},
        {10796,7707,8591,6472,3,3,1},
        {17454,10180,13322,9476,3,3,1},
        {10799,7708,8592,6473,3,3,1},
        {64657,34155,46100,12241,3,3,1},
        {6898,4777,5873,2786,3,3,1},
        {46197,26192,49285,23656,9,3,2},
        {26055,15714,21324,15680,1,3,1},
        {23214,14565,-1,14584,11,1,1},
        {23161,14505,-1,14554,11,1,1},
        {73629,39208,-1,31367,11,1,1},
        {5784,304,-1,2346,11,1,1},
        {73630,39209,-1,31368,11,1,1},
        {69826,37059,-1,30501,11,1,1},
        {69820,36883,-1,30366,11,1,1},
        {34767,20030,-1,19085,11,1,1},
        {34769,20029,-1,19296,11,1,1},
        {13819,9158,-1,8469,11,1,1},
    }
end
