local MOUNT_BUTTON_HEIGHT = 46
local PLAYER_MOUNT_LEVEL = 20

SLASH_MOUNT1 = '/mount'
SLASH_MOUNT2 = '/mnt'

local MountFavorites = CreateFrame('frame')

function MountFavorites_ToggleFavorite(spellID)
    if (MountFavoritesDB[spellID]) then
        MountFavoritesDB[spellID] = nil
    else
        MountFavoritesDB[spellID] = 1
    end
end

function MountFavorites_IsFavorite(spellID)
    return (MountFavoritesDB[spellID] == 1)
end

function MountFavorites_ToggleMount()
    for index = 1, MountJournal.mountDatabase:GetNumCompanions("MOUNT") do
        local creatureID, creatureName, spellID, icon, active, mountFlags, owned, displayInfoID, originalIndex = MountJournal.mountDatabase:GetCompanionInfo("MOUNT", index)

        if (MountFavorites.menuMountSpellID == spellID) then
            if ( active ) then
                DismissCompanion("MOUNT");
            else
                CallCompanion("MOUNT", originalIndex);
            end
        end
    end
end

function MountFavoritesMenu_ToggleFavorite()
    MountFavorites_ToggleFavorite(MountFavorites.menuMountSpellID)

    MountJournal_UpdateMountList()
end

function MountFavoritesMenu_Initialize(self, level)
    if not level then return end

    local playerLevel = UnitLevel("player");

    if level == 1 then
        local info = UIDropDownMenu_CreateInfo()

        info.disabled = nil
        info.isTitle = nil
        info.notCheckable = 1

        local spellID = MountFavorites.menuMountSpellID

        if (MountJournal.mountDatabase:FindActiveMountSpell() == spellID) then
            info.text = BINDING_NAME_DISMOUNT
        else
            info.text = MOUNT
        end

        if (playerLevel < PLAYER_MOUNT_LEVEL) then
            info.disabled = true
        end

        info.func = MountFavorites_ToggleMount

        UIDropDownMenu_AddButton(info, level)

        if (MountFavorites_IsFavorite(spellID)) then
            info.text = BATTLE_PET_UNFAVORITE
        else
            info.text = BATTLE_PET_FAVORITE
        end

        info.disabled = nil
        info.func = MountFavoritesMenu_ToggleFavorite

        UIDropDownMenu_AddButton(info, level)

        info.text = CANCEL
        info.disabled = nil
        info.func = function() end

        UIDropDownMenu_AddButton(info, level)

        MountFavorites.mountOptionsMenu = info
    end
end

function MountFavorites_UpdateCachedList(self)
    self.cachedMounts = {}

    local favorites, nonFavorites, unowned = { }, { }, { }

    for index = 1, self.mountDatabase:GetNumCompanions("MOUNT") do
        local _, _, spellID, _, _, _, owned = self.mountDatabase:GetCompanionInfo("MOUNT", index)
        if MountFavorites_MountMatchesFilter(self, index) then
            if (owned) then
                if (MountFavorites_IsFavorite(spellID)) then
                    table.insert(favorites, index)
                else
                    table.insert(nonFavorites, index)
                end
            else
                table.insert(unowned, index)
            end
        end
    end

    for _, v in pairs(favorites) do table.insert(self.cachedMounts, v) end
    for _, v in pairs(nonFavorites) do table.insert(self.cachedMounts, v) end
    for _, v in pairs(unowned) do table.insert(self.cachedMounts, v) end
end



function MountFavorites_MountMatchesFilter(self, index)
    if ( not self.searchString ) then
        return true;
    end

    local _, creatureName = self.mountDatabase:GetCompanionInfo("MOUNT", index)
    if ( string.find(string.lower(creatureName), self.searchString, 1, true) ) then
        return true;
    end

    return false;
end


function MountFavorites_UpdateMountList()
    MountFavorites_UpdateCachedList(MountJournal);

    local scrollFrame = MountJournal.ListScrollFrame;
    local offset = HybridScrollFrame_GetOffset(scrollFrame);
    local buttons = scrollFrame.buttons;


    local numMounts = GetNumCompanions("MOUNT");


    local showMounts = true;
    local playerLevel = UnitLevel("player");
    if  ( numMounts < 1 ) then
        -- display the no mounts message on the right hand side
        MountJournal.MountDisplay.NoMounts:Show();
        showMounts = false;
    else
        MountJournal.MountDisplay.NoMounts:Hide();
    end


    for i=1, #buttons do
        local button = buttons[i];
        local displayIndex = i + offset;
        if ( displayIndex <= #MountJournal.cachedMounts and showMounts ) then

            local index = MountJournal.cachedMounts[displayIndex];

            local creatureID, creatureName, spellID, icon, active, mountFlags, owned, displayInfoID = MountJournal.mountDatabase:GetCompanionInfo("MOUNT", index)

            button.name:SetText(creatureName);
            button.icon:SetTexture(icon);
            button.index = index;
            button.spellID = spellID;
            button.active = active;

            if (active and playerLevel >= PLAYER_MOUNT_LEVEL) then
                button.DragButton.ActiveTexture:Show();
            else
                button.DragButton.ActiveTexture:Hide();
            end
            button:Show();

            MountFavorites_WireUpButton(button)

            if (MountJournal.selectedSpellID == spellID) then
                button.selected = true;
                button.selectedTexture:Show();
            else
                button.selected = false;
                button.selectedTexture:Hide();
            end
            button:SetEnabled(1);
            if (playerLevel >= PLAYER_MOUNT_LEVEL and owned) then
                button.DragButton:SetEnabled(1);
                button.additionalText = nil;
                button.icon:SetDesaturated(0);
                button.icon:SetAlpha(1.0);
                button.name:SetFontObject("GameFontNormal");
            else
                button.DragButton:SetEnabled(0);
                button.additionalText = MOUNT_JOURNAL_CANT_USE;
                button.icon:SetDesaturated(1);
                button.icon:SetAlpha(.5);
                button.name:SetFontObject("GameFontDisable");
            end


            if ( button.showingTooltip ) then
                MountJournalMountButton_UpdateTooltip(button);
            end
        else
            button.name:SetText("");
            button.icon:SetTexture("Interface\\PetBattles\\MountJournalEmptyIcon");
            button.index = nil;
            button.spellID = 0;
            button.selected = false;
            button.DragButton.ActiveTexture:Hide();
            button.selectedTexture:Hide();
            button:SetEnabled(0);
            button.DragButton:SetEnabled(0);
            button.icon:SetDesaturated(1);
            button.icon:SetAlpha(0.5);
        end
    end


    --local totalHeight = #MountJournal.cachedMounts * MOUNT_BUTTON_HEIGHT;
    --HybridScrollFrame_Update(scrollFrame, totalHeight, scrollFrame:GetHeight());
    --MountJournal.MountCount.Count:SetText(numMounts);
    if ( not showMounts ) then
        MountJournal.selectedSpellID = 0;
        MountJournal_UpdateMountDisplay();
        MountJournal.MountCount.Count:SetText(0);
    end
    if ( playerLevel >= PLAYER_MOUNT_LEVEL and numMounts > 0) then
        MountJournal.MountButton:SetEnabled(1);
    else
        MountJournal.MountButton:SetEnabled(0);
    end
end

function MountFavorites_FindSelectedIndex()
    local selectedSpellID = MountJournal.selectedSpellID;
    if ( selectedSpellID ) then
        for i=1, MountJournal.mountDatabase:GetNumCompanions("MOUNT") do
            local creatureID, creatureName, spellID, icon, active = MountJournal.mountDatabase:GetCompanionInfo("MOUNT", i);
            if ( spellID == selectedSpellID ) then
                return i
            end
        end
    end

    return nil
end

function MountFavorites_UpdateMountDisplay()
    local index = MountFavorites_FindSelectedIndex()

    if ( index ) then

        local creatureID, creatureName, spellID, icon, active, mountFlags, owned, displayInfoID = MountJournal.mountDatabase:GetCompanionInfo("MOUNT", index)

        if ( MountJournal.MountDisplay.realLastDisplayed ~= spellID ) then
            MountJournal.MountDisplay.Name:SetText(creatureName);
            --MountJournal.MountDisplay.ModelFrame:SetCreature(creatureID);
            MountJournal.MountDisplay.realLastDisplayed = spellID;
        end

        if (displayInfoID) then
            MountJournal.MountDisplay.ModelFrame:SetDisplayInfo(displayInfoID)
        end

        MountJournal.MountDisplay.Name:Show();
        MountJournal.MountDisplay.ModelFrame:Show();
        MountJournal.MountDisplay.YesMountsTex:Show();
        MountJournal.MountDisplay.NoMountsTex:Hide();
        MountJournal.MountDisplay.NoMounts:Hide();

        if ( active ) then
            MountJournal.MountButton:SetText(BINDING_NAME_DISMOUNT);
        else
            MountJournal.MountButton:SetText(MOUNT);
        end
    else
        MountJournal.MountDisplay.Name:Hide();
        MountJournal.MountDisplay.ModelFrame:Hide();
        MountJournal.MountDisplay.YesMountsTex:Hide();
        MountJournal.MountDisplay.NoMountsTex:Show();
        MountJournal.MountDisplay.NoMounts:Show();
    end
end

function MountFavorites_ShowMountDropdown(anchorTo, offsetX, offsetY, spellID)
    MountFavorites.menuMountSpellID = spellID

    ToggleDropDownMenu(1, nil, MountFavorites_DropDownMenu, anchorTo, offsetX, offsetY)
end

function MountFavorites_HideMountDropdown()
    if (UIDropDownMenu_GetCurrentDropDown() == MountFavorites_DropDownMenu) then
        HideDropDownMenu(1);
    end
end

function MountFavorites_WireUpButton(button)
    if (not button.DragButton.favorite) then
        local texture = button:CreateTexture('favorite', 'OVERLAY')

        texture:SetSize(25,25)
        texture:SetPoint('TOPLEFT', button.icon, 'TOPLEFT', -8, 8)
        texture:SetTexture('Interface/PetBattles/PetJournal')
        texture:SetTexCoord(0.11328125, 0.16210938, 0.02246094, 0.04687500)

        button:SetScript("OnClick", function(self, button, down)
            if button == "RightButton" then
                MountFavorites_ShowMountDropdown(self, 80, 20, self.spellID)
            elseif button == "LeftButton" then
                MountListItem_OnClick(self, button)
            end
        end)

        button.DragButton.favorite = texture
    end

    if (MountFavorites_IsFavorite(button.spellID)) then
        button.DragButton.favorite:Show()
    else
        button.DragButton.favorite:Hide()
    end

end

function MountFavorites_SlashMountCommand(msg, editBox)
    if msg == 'builddb' then
        --MountDatabase:BuildFromPlayerMounts();
    end
end

function MountFavorites_FindActiveMountSpell()
    for i = 1, GetNumCompanions("MOUNT") do
        local _, _, spellID, _, active = GetCompanionInfo("MOUNT", i);
        if (active) then
            return spellID
        end
    end

    return nil
end

function MountFavorites_Select(index)
    local _, _, spellID = MountJournal.mountDatabase:GetCompanionInfo("MOUNT", index)

    MountJournal.selectedSpellID = spellID
    MountJournal_UpdateMountList()
    MountJournal_UpdateMountDisplay()
end

MountFavorites:SetScript('OnEvent', function(self, event, name)
	if name == 'MountFavorites' then
        -- saved variables are available now
        if (MountFavoritesDB == nil) then
            MountFavoritesDB = {}
        end

        SlashCmdList['MOUNT'] = MountFavorites_SlashMountCommand

        local addonName = GetAddOnMetadata('MountFavorites', 'Title')
        local addonVersion = GetAddOnMetadata('MountFavorites', 'Version')

        print(addonName .. ' ' .. addonVersion .. ' loaded')
	elseif name == 'Blizzard_PetJournal' and event == 'ADDON_LOADED' then
        self:UnregisterEvent('ADDON_LOADED')

        CreateFrame("Frame", "MountFavorites_DropDownMenu", MountJournal, "UIDropDownMenuTemplate")
        MountFavorites_DropDownMenu.displayMode = "MENU"
        MountFavorites_DropDownMenu.initialize = MountFavoritesMenu_Initialize

        hooksecurefunc("MountJournal_UpdateMountList", MountFavorites_UpdateMountList)
        hooksecurefunc("MountJournal_UpdateMountDisplay", MountFavorites_UpdateMountDisplay)
        hooksecurefunc("MountJournal_Select", MountFavorites_Select)
        hooksecurefunc(MountJournal.ListScrollFrame, 'SetVerticalScroll', MountFavorites_UpdateMountList)

    elseif event == 'COMPANION_LEARNED' then
        MountFavorites_HideMountDropdown()
    elseif event == 'COMPANION_UPDATE' then
        if MountJournal.mountDatabase == nil then
            MountJournal.mountDatabase = CreateMountDatabase()
        end

        MountFavorites_HideMountDropdown()
	end
end)

MountFavorites:RegisterEvent('ADDON_LOADED')
MountFavorites:RegisterEvent("COMPANION_LEARNED")
MountFavorites:RegisterEvent("COMPANION_UNLEARNED")
MountFavorites:RegisterEvent("COMPANION_UPDATE")
